# This file is part of DesktopDig
# Copyright (C) 2002 Daniel Naber <daniel.naber@t-online.de>

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA

import os
import re

class IgnoreTerms:

	def __init__(self, basedir, min_length, max_length):
		self.filename = os.path.join(basedir, "etc/stopwords_en.txt")
		self.min_length = min_length
		self.max_length = max_length
		f = open(self.filename)
		contents = f.read()
		f.close()
		self.stopwords = {}
		# use a dict for fast lookup:
		for stopword in re.split("[\n\r]", contents):
			self.stopwords[stopword] = 1		# 1 is a fake value
		#print "Loaded %d stopwords." % len(self.stopwords.keys())
		return

	def isStopTerm(self, word):
		if self.stopwords.has_key(word):
			return 1
		else:
			return 0

	def acceptTerm(self, term):
		"""Don't accept terms (=return None) that are too short or that are part of the
		stop word list. Words that are too log are shortened and the short version is
		returned."""
		if len(term) < self.min_length:
			return None
		elif len(term) > self.max_length:
			term = term[:self.max_length]
		if self.isStopTerm(term):
			#print "Ignoring stopword '%s'" % term
			return None
		return term
