#!/usr/bin/python
# This file is part of DesktopDig
# Copyright (C) 2002 Daniel Naber <daniel.naber@t-online.de>

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA

import os 
import re
import sys

import FullTextIndex
import Search

import unittest

class SearchTest(unittest.TestCase):

	def setUp(self):
		self.idx = FullTextIndex.FullTextIndex(basedir, "backend/test_db/one_test", 0,
			2, 15, 0)
		self.idx.deleteData()
		self.idx.bindData()
		self.idx.addDocument(None, "bla sabbel blubb", title="A title")
		self.idx.addDocument(None, "xxx blubb", title="B title")
		self.idx.addDocument(None, "eins blubb bla sabbel zwei drei", title="C title")
		self.idx.finishIndex()
		self.idx.calculateRanking()
		self.idx.commitData()
		return
	
	#def testExceptions(self):
	#	self.assertRaises(Search.UnknownLogicalOperator, Search.Search, basedir, "bla", "foobar!")
	#	return

	def testGetAndResults(self):

		res = self.getResults("foofoo nonono", 'and')
		self.assertEqual(len(res), 0)

		# single term:
		res = self.getResults("bla", 'and')
		self.assertEqual(len(res), 2)
		#print "###%s" % str(res[0].relevancy)
		#print "###%s" % res[1].relevancy
		self.assertEqual(res[0].doc.doc_id, 1)
		self.assertEqual(res[1].doc.doc_id, 3)

		res = self.getResults("bla sabbel", 'and')
		self.assertEqual(len(res), 2)
		self.assertEqual(res[0].doc.doc_id, 1)
		self.assertEqual(res[1].doc.doc_id, 3)

		res = self.getResults("bla sabbel xxx", 'and')
		self.assertEqual(len(res), 0)

		res = self.getResults("blubb xxx", 'and')
		self.assertEqual(len(res), 1)
		self.assertEqual(res[0].doc.doc_id, 2)

		res = self.getResults("drei zwei eins", 'and')
		self.assertEqual(len(res), 1)
		self.assertEqual(res[0].doc.doc_id, 3)

	def testGetOrResults(self):

		res = self.getResults("foofoo nonono", 'or')
		self.assertEqual(len(res), 0)

		# single term:
		res = self.getResults("bla", 'or')
		self.assertEqual(len(res), 2)
		self.assertEqual(res[0].doc.doc_id, 1)
		self.assertEqual(res[1].doc.doc_id, 3)

		res = self.getResults("bla sabbel", 'or')
		self.assertEqual(len(res), 2)
		self.assertEqual(res[0].doc.doc_id, 1)
		self.assertEqual(res[1].doc.doc_id, 3)

		res = self.getResults("bla nonono", 'or')
		self.assertEqual(len(res), 2)
		self.assertEqual(res[0].doc.doc_id, 1)
		self.assertEqual(res[1].doc.doc_id, 3)

		# order mustn't matter:
		res = self.getResults("nonono bla", 'or')
		self.assertEqual(len(res), 2)
		self.assertEqual(res[0].doc.doc_id, 1)
		self.assertEqual(res[1].doc.doc_id, 3)

		res = self.getResults("bla blubb", 'or')
		self.assertEqual(len(res), 3)
		self.assertEqual(res[0].doc.doc_id, 1)
		self.assertEqual(res[1].doc.doc_id, 3)	# bla AND blubb, so higher relevancy than doc 2
		self.assertEqual(res[2].doc.doc_id, 2)

		res = self.getResults("bla xxx eins", 'or')
		self.assertEqual(len(res), 3)

		###
		#self.idx.addDocument(None, "bla sabbel blubb", title="A title")
		#self.idx.addDocument(None, "xxx blubb", title="B title")
		#self.idx.addDocument(None, "eins blubb bla sabbel", title="C title")

		#for d in res:
		#	print "###%s %s" % (d.relevancy, d.doc.title)

		self.assertEqual(res[0].doc.doc_id, 2)
		# TODO: maybe this should be at top, but the fact that the document
		# is longer decreases the relevance:
		self.assertEqual(res[1].doc.doc_id, 3)	# bla AND eins -> higher relevancy
		self.assertEqual(res[2].doc.doc_id, 1)

		return

	def getResults(self, query, operator):
		s = Search.Search(basedir, query, "backend/test_db/one_test", 2, 20, 0)
		res = []
		if operator == 'and':
			res = s.getAndResults()
		elif  operator == 'or':
			res = s.getOrResults()
		else:
			print "Error: Unknown operator."
			sys.exit(1)
		res.sort()
		return res
	
# the base directory is always the top directory (containing "index.py" etc.):
basedir = os.path.abspath(os.path.split(sys.argv[0])[0])
basedir_match = re.compile("(.*)/.*?").match(basedir)
basedir = basedir_match.group(1)

if __name__ == "__main__":
    unittest.main()
