#!/usr/local/bin/python
# Tests for Set.py.
# This file is part of DesktopDig
# Copyright (C) 2002 Daniel Naber <daniel.naber@t-online.de>

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA

import unittest
import Set

class SetTest(unittest.TestCase):

	def testAdd(self):
		s = Set.Set()
		assert(s.asList() == [])
		s.add(5)
		assert(s.asList() == [5])
		# no duplicates:
		s.add(5)
		assert(s.asList() == [5])
		s.add(1)
		assert(s.asList() == [1,5])
		s.add(100)
		assert(s.asList() == [1,5,100])
		s.add("test")
		assert(s.asList() == [1,5,100,'test'])
		s.add("zz")
		assert(s.asList() == [1,5,100,'test','zz'])

	def testAddList(self):
		s = Set.Set()
		s.addList([])
		assert(s.asList() == [])
		s.addList([5,5,7,1])
		assert(s.asList() == [1,5,7])
		s.addList([5,5,7,1])
		assert(s.asList() == [1,5,7])

	def testRemove(self):
		s = Set.Set()
		s.addList([5,5,7,1])
		s.remove(5)
		assert(s.asList() == [1,7])
		assert(s.asList() != [1,5,7])
		s.remove(1)
		s.remove(7)
		assert(s.asList() == [])

	def testHasElement(self):
		s = Set.Set()
		s.addList([5,5,7,1])
		assert(s.hasElement(5))
		assert(s.hasElement(7))
		assert(s.hasElement(1))
		assert(not s.hasElement(2))
		s.remove(1)
		assert(not s.hasElement(1))

	def testUnion(self):
		s = Set.Set()
		s2 = Set.Set()
		assert(s.union(s2).asList() == [])
		s.addList([7,8])
		assert(s.union(s2).asList() == [7,8])
		s2.addList([7])
		assert(s.union(s2).asList() == [7,8])
		s2.addList([10])
		assert(s.union(s2).asList() == [7,8,10])

	def testIntersection(self):
		s = Set.Set()
		s2 = Set.Set()
		assert(s.intersection(s2).asList() == [])
		s.addList([7,8])
		assert(s.intersection(s2).asList() == [])
		s.addList([7,8])
		s2.addList([7])
		assert(s.intersection(s2).asList() == [7])
		s.addList([7,8,6,5,4])
		s2.addList([7,6,5,4])
		assert(s.intersection(s2).asList() == [4,5,6,7])

	def testCombination(self):
		s = Set.Set()
		s2 = Set.Set()
		s3 = Set.Set()
		s.addList([7,8])
		s2.addList([8,9,10,7,11])
		s3.addList(s2.asList())
		# everything expect intersection:
		inters = s.intersection(s2)		# = [7,8]
		assert(s3.removeSet(inters).asList() == [9,10,11])

if __name__ == "__main__":
    unittest.main()
