# This file is part of DesktopDig
# Copyright (C) 2002 Daniel Naber <daniel.naber@t-online.de>

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA

import re
import string
 
class Template:
 
	def __init__(self, template_file):
		template = open(template_file, "r")
		self.content = template.read()
		template.close()
		return
 
	def substitute(self, **params):
		"""Insert values into the template."""
		for arg in params:
			if not params[arg]:
				params[arg] = ""
			replaceString = "__%s__" % arg
			self.content = string.replace(self.content, \
				replaceString, params[arg])
		return self.content

	def populateLoop(self, name, **params):
		"""Insert values into the loop. The loop template is kept 
		intact so this can be called again."""
		loop_template_match = \
			re.compile("__loop:%s__(.*)__end:%s__" % (name, name), \
			re.MULTILINE|re.DOTALL).search(self.content)
		loop_template = loop_template_match.group(1)
		#print "content-type: text/html\n\n##"+str(loop_template)+"##"
		for arg in params:
			replaceString = "__item:%s__" % arg
			loop_template = string.replace(loop_template, \
				replaceString, params[arg])
		#print "content-type: text/html\n\n" + "###ANF"	 + str(loop_template) +"###ENDE"
		replaceString = "__loop:%s__" % name
		newString = "%s%s" % (loop_template, replaceString)
		self.content = string.replace(self.content, \
			replaceString, newString)
		return self.content

	def finalizeLoop(self, name):
		"""Remove the loop template code."""
		regex = re.compile("__loop:%s__(.*)__end:%s__" % (name, name), \
			re.MULTILINE|re.DOTALL)
		self.content = regex.sub("", self.content)
		return self.content
