#!/usr/bin/python
# This file is part of DesktopDig
# Copyright (C) 2002 Daniel Naber <daniel.naber@t-online.de>

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA

#time ./index.py /data/bigindex/doc/ (2300 files) 6:14, 23MB RAM 

import getopt
import os
import sys

import Config
import Indexer

def usage():
	print "Usage: index.py [-h|--help] [-u|--update] [-n|--name indexname] <directory...>"
	# fixme: explain options
	return

def main():
	options = None
	rest = None
	index_suffix = None
	try:
		(options, rest) = getopt.getopt(sys.argv[1:], 'hun:', ['help', 'update', 'name='])
	except getopt.GetoptError, e:
		print "Error: %s" % e
		usage()
		sys.exit(1)
	update_mode = 0
	for o, a in options:
		if o in ("-h", "--help"):
			usage()
			sys.exit(0)
		elif o in ("-u", "--update"):
			update_mode = 1
		elif o in ("-n", "--name"):
			index_suffix = a
	if not rest:
		usage()
		sys.exit(1)

	cfg = Config.Config()
	indexer = Indexer.Indexer(Config.BASEDIR, update=update_mode, \
		suffixes=cfg.getList('SUFFIXES'),
		desc_words=cfg.getInt('DESC_WORDS'),
		min_length=cfg.getInt('MIN_LENGTH'),
		max_length=cfg.getInt('MAX_LENGTH'),
		phrase_search=cfg.getInt('PHRASE_SEARCH'), 
		default_encoding=cfg.getString('DEFAULT_ENCODING'),
		files_world_readable=cfg.getInt('FILES_WORLD_READABLE'))
	index_name = "data/index"
	if index_suffix:
		# fixme: check that index_suffix =~ ^[\w]+$
		index_name = "%s_%s" % (index_name, index_suffix)

	information = indexer.start(directories=rest, index_name=index_name)
	
	print "Files indexed = %d" % information['files']
	print "Size of all files indexed = %.2f kb" % (information['files_size']/1024.0)
	print "Average file size = %.2f kb" % information['average_file_size']
	#print indexer.files_per_directory

main()
