#!/usr/bin/python
# Copyright (C) 2002,2003 Daniel Naber <daniel.naber@t-online.de>

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA

import cgi
import cgitb
import copy
import getopt
import os
import re
import string
import sys
import time
import urllib

import Config
import Search
import Template

cgitb.enable()		# debugging only

def escapeHTML(s):
	if not s:
		return None
	s = string.replace(s, "&", "&amp;")
	s = string.replace(s, "<", "&lt;")
	s = string.replace(s, ">", "&gt;")
	s = string.replace(s, '"', "&quot;")
	s = string.replace(s, "'", "&apos;")
	return s
	
def ceil(x, y):
	if x % y == 0:
		return x / y
	else:
		return int(x / y + 1)

def formatResults(matches, template, first, search, cfg):
	res = []
	i = 1
	for match in matches:
		date = time.strftime(cfg.getString('TIME_FORMAT'), time.localtime(match.doc.date))
		size = "%.0f" % (match.doc.size/1024.0)
		title = match.doc.title
		score = "%.1f" % match.relevancy
		desc = "(No description available)"
		if not title:
			title = match.doc.url
		if title == "":
			title = "Untitled"
		if match.doc.description:
			desc = match.doc.description
		url = urllib.quote(match.doc.url)
		url = "/%s" % url.replace(cfg.getString('CUT_PATH'), "")
		url = "%s%s" % (cfg.getString('BASEURL'), url)
		html_res = template.populateLoop("results",
			rank=str(first+i),
			url=url,
			visibleurl=escapeHTML(url),
			title=search.highlightTerms(title),
			description=search.highlightTerms(desc),
			score=str(score),
			date=date,
			size=size
			)
		i = i + 1
	return template.content

def main():
	start_time = time.time()
	form = cgi.FieldStorage()

	search_type = 'and'
	match_all_check = ''
	match_any_check = ''
	if form.getvalue("mode") == 'all':
		search_type = 'and'
		match_all_check = 'selected="selected"'
	else:
		search_type = 'or'
		match_any_check = 'selected="selected"'
	
	# TODO: this assumes the form page is utf-8:
	if form.getvalue("q"):
		query_string = form.getvalue("q").decode('utf-8')
	else:
		query_string = ""
	if not query_string:
		query_string = ""

	### Search:

	s = None
	index_name = "data/index"
	if form.getvalue("collection"):
		if not re.match("^[a-zA-Z0-9]+$", form.getvalue("collection")):
			print "Content-Type: text/html\n"
			print "Error: Illegal index name '%s'" % escapeHTML(form.getvalue("collection"))
			sys.exit(1)
		index_name = "%s_%s" % (index_name, form.getvalue("collection"))
	cfg = Config.Config()
	try:
		s = Search.Search(Config.BASEDIR, query_string, index_name,
			cfg.getInt('MIN_LENGTH'), cfg.getInt('MAX_LENGTH'),
			cfg.getInt('PHRASE_SEARCH'))
	except IOError, e:
		print "Content-Type: text/html\n"
		print "Error: Index could not be found or could not be read."
		print "You might have to set <tt>files_world_readable = 1</tt>"
		print "in <tt>desktopdig.config</tt> and re-index your files."
		sys.exit(1)
		
	if search_type == 'or':
		matches = s.getOrResults()
	else:
		matches = s.getAndResults()

	# Filter:

	include = ''
	if form.getvalue("include"):
		include = form.getvalue("include")
		matches = [ m for m in matches if (re.search(re.escape(include), m.doc.url)) ]

	exclude = ''
	if form.getvalue("exclude"):
		exclude = form.getvalue("exclude")
		matches = [ m for m in matches if (not re.search(re.escape(exclude), m.doc.url)) ]

	docs_total = s.countDocuments()

	current_page = 1
	if form.getvalue("p"):
		current_page = int(form.getvalue("p"))
	first = (current_page - 1) * cfg.getInt('RESULTS_PER_PAGE')
	last = first + cfg.getInt('RESULTS_PER_PAGE') - 1
	real_last = len(matches)
	if last >= real_last:
		last = real_last - 1
	displayed_matches = matches[first:last+1]

	last_page = ceil(real_last, cfg.getInt('RESULTS_PER_PAGE'))
	if last_page < 1:
		last_page = 1

	### Build the URLs:

	queries = ''
	if form.getvalue("include"):
		queries = "&amp;include=%s" % urllib.quote(form.getvalue("include"));
	if form.getvalue("exclude"):
		queries += "&amp;exclude=%s" % urllib.quote(form.getvalue("exclude"));
	if form.getvalue("mode"):
		queries += "&amp;mode=%s" % urllib.quote(form.getvalue("mode"));
	if form.getvalue("collection"):
		queries += "&amp;collection=%s" % urllib.quote(form.getvalue("collection"));
	if form.getvalue("q"):
		queries += "&amp;q=%s" % urllib.quote(form.getvalue("q"));

	### Build the page navigation:
	
	previous_html = ''
	next_html = ''
	if current_page == 1:
		if last_page > current_page:
			next_html = '<a href="%s?p=2%s">%s</a>' % \
				(cfg.getString('SEARCH_URL'), queries, cfg.getString('NEXT_PAGE_TEXT'))
	elif current_page == last_page:
		previous_html = '<a href="%s?p=%d%s">%s</a>' % \
			(cfg.getString('SEARCH_URL'), last_page-1, queries, cfg.getString('PREV_PAGE_TEXT'))
	else:
		previous_html = '<a href="%s?p=%d%s">%s</a>' % \
			(cfg.getString('SEARCH_URL'), current_page-1, queries, cfg.getString('PREV_PAGE_TEXT'))
		next_html = '<a href="%s?p=%d%s">%s</a>' % \
			(cfg.getString('SEARCH_URL'), current_page+1, queries, cfg.getString('NEXT_PAGE_TEXT'))

	navbar = []
	for i in range(1, last_page+1):
		if i != current_page:
			navbar.append("<a href=\"%s?p=%d%s\">%d</a> " % \
				(cfg.getString('SEARCH_URL'), i, queries, i))
		else:
			navbar.append("<strong>%d</strong>" % i)
		i = i + 1
	navbar_html = string.join(navbar, ' ')

	### Populate the template:

	filename = ''
	if len(matches) == 0:
		filename = os.path.join(Config.BASEDIR, "templates/no_match.html")
	else:
		filename = os.path.join(Config.BASEDIR, "templates/results.html")
	
	html_template = Template.Template(filename)
	end_time = time.time()
	run_time = end_time - start_time

	ignored_terms = ''
	if s.ignored_tokens:
		tokens = []
		for t in s.ignored_tokens:
			tokens.append(escapeHTML(t))
		ignored_terms = cfg.getString('IGNORED_TEXT') % string.join(tokens, ', ')

	# TODO: make this less hacky:
	index_update = os.stat(os.path.join(Config.BASEDIR, "%s_docs_bee.dat" % index_name)).st_mtime
	index_update = time.strftime(cfg.getString('TIME_FORMAT'), time.localtime(index_update))

	html_template.substitute(
		query_str=escapeHTML(query_string),
		query_str_escaped=urllib.quote(query_string),
		results_num=str(len(matches)),
		search_time= "%.2f" % (run_time),
		current_page=str(current_page),
		first_number=str(first+1),
		last_number=str(last+1),
		previous=previous_html,
		navbar=navbar_html,
		next=next_html,
		total_pages=str(last_page),
		baseurl=cfg.getString('BASEURL'),
		docs_total=str(docs_total),
		search_url=cfg.getString('SEARCH_URL'),
		include=escapeHTML(include),
		exclude=escapeHTML(exclude),
		collection=escapeHTML(form.getvalue("collection")),
		match_all_check=match_all_check,
		match_any_check=match_any_check,
		ignored_terms=ignored_terms,
		index_update=index_update
		)
	formatResults(displayed_matches, html_template, first, s, cfg)
	res = html_template.finalizeLoop("results")
	print "Content-Type: text/html; charset=utf-8\n"
	print res.encode('utf-8')
	return

main()
