#!/usr/bin/python
# This file is part of DesktopDig
# Copyright (C) 2002,2003 Daniel Naber <daniel.naber@t-online.de>

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
# USA

import getopt
import os
import string
import sys
import time

import Config
import Search

def usage():
	print "Usage: search.py [-h|--help] [-a|--and, -o|--or] [-n|--name indexname] <term...>"
	# fixme: explain options
	return

def main():
	options = None
	rest = None
	index_suffix = None
	try:
		(options, rest) = getopt.getopt(sys.argv[1:], 'haon:', ['help', 'and', 'or', 'name='])
	except getopt.GetoptError, e:
		print "Error: %s" % e
		usage()
		sys.exit(1)
	search_type = 'and'
	for o, a in options:
		if o in ("-h", "--help"):
			usage()
			sys.exit(0)
		elif o in ("-a", "--and"):
			search_type = 'and'
		elif o in ("-o", "--or"):
			search_type = 'or'
		elif o in ("-n", "--name"):
			index_suffix = a
	if not rest:
		usage()
		sys.exit(1)
	query = string.join(rest, ' ').decode('latin1')		# TODO: configurable?
	cfg = Config.Config()
	index_name = "data/index"
	if index_suffix:
		index_name = "%s_%s" % (index_name, index_suffix)
	s = Search.Search(Config.BASEDIR, query, index_name, \
		cfg.getInt('MIN_LENGTH'), \
		cfg.getInt('MAX_LENGTH'),
		cfg.getInt('PHRASE_SEARCH'))
	if search_type == 'and':
		matches = s.getAndResults()
	elif search_type == 'or':
		matches = s.getOrResults()
	else:
		print "Error: Unknown operator '%s'" % search_type
	res = formatResults(matches, query)
	print res.encode('latin1')	# TODO: configurable?

def formatResults(matches, query):
	res = []
	i = 1
	for match in matches:
		date = time.strftime("%Y-%m-%d %H:%M", time.localtime(match.doc.date))
		res.append("%d. %s Title: %s Date: %s Relevancy: %.2f" %
			(i, match.doc.url, match.doc.title, date, match.relevancy))
		i = i + 1
	if len(res) == 0:
		res.append("No matches for '%s'." % query)
	return string.join(res, '\n')

#import profile
#profile.run('main()')
main()
